import sqlite3
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import Column, Integer, String, Boolean, Text, create_engine, ForeignKey
from sqlalchemy.orm import sessionmaker
from sqlalchemy.sql import text
from server import fs_db

from passlib.context import CryptContext
import math
import random
import json

Base = declarative_base()
engine = create_engine('sqlite:///database.db')
Base.metadata.bind = engine

class User(Base):
  __tablename__ = 'user'
  id = Column(Integer, primary_key=True)
  first_name = Column(String(60), nullable=False)
  last_name = Column(String(60), nullable=False)
  gender = Column(String(10))
  city = Column(String(60))
  country = Column(String(60))
  email = Column(Text(), nullable=False)
  password = Column(String(60), nullable=False)
  token = Column(Text())

class Message(Base):
  __tablename__ = 'message'
  id = Column(Integer, primary_key=True)
  sender_id = Column(String(60), ForeignKey(User.id), nullable=False)
  receiver_id = Column(Integer, ForeignKey(User.id), nullable=False)
  message = Column(Text())
  location = Column(Text())

# Init table
Base.metadata.create_all(engine)

def createSession():
  DBSession = sessionmaker(bind=engine)
  return DBSession()

def GenerateToken():
  letters = "abcdefghiklmnopqrstuvwwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"
  token = ""
  for i in range(36):
    token = token + letters[math.floor(random.random() * len(letters))]
  return token

def CreateUser(request):
  session = fs_db.session
  myctx = CryptContext(schemes=["sha256_crypt"])
  new_user = User(
    first_name=request['firstname'],
    last_name=request['familyname'],
    gender=request['gender'],
    city=request['city'],
    country=request['country'],
    email=request['email'],
    password=myctx.hash(request['password'])
  )
  session.add(new_user)
  session.commit()
  return None

def CheckEmailNotExist(email):
  session = fs_db.session
  if session.query(User).filter(User.email == email).first() is None:
    return True
  else:
    return False

def Login(request):
  session = fs_db.session
  myctx = CryptContext(schemes=["sha256_crypt"])
  user = session.query(User).filter(User.email == request['email']).first()
  if user is None:
    return False
  else:
    return myctx.verify(request['password'], user.password)

def CreateToken(email):
  session = fs_db.session
  user = session.query(User).filter(User.email == email).first()
  if user is None:
    return None
  else:
    token = GenerateToken()
    user.token = token
    session.commit()
    return token

def CheckTokenExists(token):
  session = fs_db.session
  if session.query(User).filter(User.token == token).first() is not None:
    return True
  else:
    return False

def DeleteToken(token):
  session = fs_db.session
  user = session.query(User).filter(User.token == token).first()
  if user is None:
    return False
  else:
    user.token = ""
    session.commit()
    return True

def ChangePassword(request, token):
  session = fs_db.session
  myctx = CryptContext(schemes=["sha256_crypt"])
  user = session.query(User).filter(User.token == token).first()  
  if myctx.verify(request['password'], user.password):
    user.password = myctx.hash(request['new_password'])
    session.commit()
    return True
  else:
    return False

def ShowUserInfoByToken(token):
  session = fs_db.session
  user = session.query(User).filter(User.token == token).first()
  if user is not None:
    return user.__dict__
  else:
    return None

def ShowUserInfoByEmail(email):
  session = fs_db.session
  user = session.query(User).filter(User.email == email).first()
  if user is not None:
    return user.__dict__
  else:
    return None

def ShowUserIdByEmail(email):
  session = fs_db.session
  user = session.query(User).filter(User.email == email).first()
  if user is not None:
    return user.id
  else:
    return None

def ShowUserIdByToken(token):
  session = fs_db.session
  user = session.query(User).filter(User.token == token).first()
  if user is not None:
    return user.id
  else:
    return None

def CreateMessage(sender_id, receiver_id, message, location):
  session = fs_db.session
  new_msg = Message(
    receiver_id=receiver_id,
    sender_id=sender_id,
    message=message,
    location=location
  )
  session.add(new_msg)
  session.commit()
  return True

def ShowMessageByToken(token):
  session = fs_db.session
  statement = text("""
    select rcd.email as rcd_email, u.email as sender_email, rcd.message as message, rcd.location as location
    from user as u
    join 
    (
      select u.id as uid, u.email as email, m.sender_id as sender_id, m.message as message, m.location as location
      from user as u, message as m
      where u.id = m.receiver_id
      and u.token = :token
    ) as rcd
    on rcd.sender_id = u.id
  """)
  result = session.execute(statement, {'token': token})
  return row2dict(result)

def row2dict(rows):
  result = []
  for row in rows:
    result.append(dict(row))
  return result


def ShowMessageByEmail(email):
  session = fs_db.session
  statement = text("""
    select rcd.email as rcd_email, u.email as sender_email, rcd.message as message, rcd.location as location
    from user as u
    join 
    (
      select u.id as uid, u.email as email, m.sender_id as sender_id, m.message as message, m.location as location
      from user as u, message as m
      where u.id = m.receiver_id
      and u.email = :email
    ) as rcd
    on rcd.sender_id = u.id
  """)
  result = session.execute(statement, {'email': email})
  return row2dict(result)

def test():
  session = fs_db.session
  user = session.query(User).filter(User.token == 'a@a.com').first()
  if user is None:
    return None
  else:
    token = GenerateToken()
    user.token = token
    session.commit()
    return token


if __name__== "__main__":
# user = {
#   "receiver_id": '1',
#   "sender_id": '2',
#   "message": 'Yeee',
# }
  # print(CreateMessage(user))
  print(ShowUserIdByToken("UiDR1RXNDWMP5JARrCAbUnvqU3aCvhwB1DK9"))
  # print(test())
  print("Hello World!")

