import sys, os
sys.path.append(os.path.join(os.path.dirname(__file__)))

import flask
import json
from flask import render_template, request, jsonify
from flask_sqlalchemy import SQLAlchemy
import database_helper as db
from gevent.pywsgi import WSGIServer
from geventwebsocket.handler import WebSocketHandler

app = flask.Flask(__name__, template_folder='.', static_folder='static')
app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///database.db'
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
fs_db = SQLAlchemy(app)

logged = {}

def GenerateToken():
  letters = "abcdefghiklmnopqrstuvwwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890"
  token = ""
  for i in range(36):
    token = token + letters[math.floor(random.random() * len(letters))]
  return token

@app.route('/', methods=['GET'])
@app.route('/index', methods=['GET'])
def index():
  return render_template('client.html')

@app.route('/index', methods=['POST'])
def signUp():
  response = {
    'status': '404',
    'success': False,
    'message': '',
  }
  data = request.json
  if (
    type(data['email'] is str)
    (type(data['email']) is str) and
    (type(data['password']) is str) and
    (type(data['firstname']) is str) and
    (type(data['familyname']) is str) and
    (type(data['gender']) is str) and
    (type(data['city']) is str) and
    (type(data['country']) is str)
  ):
    if db.CheckEmailNotExist(data['email']):
      db.CreateUser(data)
      response['success'] = True
      response['status'] = '200'
      response['message'] = "Successfully created a new user."
    else:
      response['success'] = False
      response['message'] = "Email already exists."  
  else:
    response['success'] = False
    response['message'] = "Form data missing or incorrect type."  

  return jsonify(response)

@app.route('/signIn', methods=['POST'])
def signIn():
  response = {
    'status': '404',
    'success': False,
    'message': '',
    'token': '',
  }
  data = request.json
  if (type(data['email']) is str) and (type(data['password']) is str):
    if db.Login(data):
      if data['email'] in logged:
        msg = {
          'message': 'Same user logged in.'
        }
        try:
          logged[data['email']].send(json.dumps(msg))
        except:
          pass
      token = db.CreateToken(data['email'])
      response['success'] = True
      response['status'] = '200'
      response['message'] = "Successfully login a user."
      response['token'] = token
    else:
      response['success'] = False
      response['message'] = "Email or password is not correct."  
  else:
    response['success'] = False
    response['message'] = "Form data missing or incorrect type."  

  return jsonify(response)

@app.route('/signOut', methods=['POST'])
def signOut():
  response = {
    'status': '',
    'success': False,
    'message': ''
  }
  token = request.headers.get("token")  
  if db.CheckTokenExists(token):
    if db.DeleteToken(token):
      response['success'] = True
      response['status'] = '200'
      response['message'] = "Successfully signed out."
    else:
      response['success'] = False
      response['message'] = "Delete token errors."
  else:
    response['success'] = False
    response['message'] = "You are not signed in."

  return jsonify(response)

@app.route('/changePassword', methods=['POST'])
def changePassword():
  response = {
    'status': '',
    'success': False,
    'message': ''
  }
  token = request.headers.get("token")
  data = request.json
  if db.CheckTokenExists(token):
    if db.ChangePassword(data, token):
      response['success'] = True
      response['status'] = '200'
      response['message'] = "Successfully changed."
    else:
      response['success'] = False
      response['message'] = "Password incorrect."
  else:
    response['success'] = False
    response['message'] = "You are not signed in."

  return jsonify(response)

@app.route('/showUserInfoByToken', methods=['POST'])
def showUserInfoByToken():
  response = {
    'status': '',
    'success': False,
    'message': ''
  }
  token = request.headers.get("token")
  user = db.ShowUserInfoByToken(token)
  if user is not None:
    response['success'] = True
    response['status'] = '200'
    response['email'] = user['email']
    response['firstname'] = user['first_name']
    response['familyname'] = user['last_name']
    response['country'] = user['country']
    response['city'] = user['city']
    response['gender'] = user['gender']
  else:
    response['success'] = False
    response['message'] = "Token incorrect."

  return jsonify(response)

@app.route('/showUserInfoByEmail', methods=['POST'])
def showUserInfoByEmail():
  response = {
    'status': '',
    'success': False,
    'message': '',
  }
  token = request.headers.get("token")
  data = request.json
  if db.CheckTokenExists(token):
    user = db.ShowUserInfoByEmail(data['email'])
    if user is not None:
      status = 200
      response['success'] = True
      response['email'] = user['email']
      response['firstname'] = user['first_name']
      response['familyname'] = user['last_name']
      response['country'] = user['country']
      response['city'] = user['city']
      response['gender'] = user['gender']
    else:
      status = 400
      response['success'] = False
      response['message'] = "Email incorrect."
  else:
    status = 400
    response['success'] = False
    response['message'] = "Token invalid."

  return jsonify(response), status

@app.route('/showMessageByToken', methods=['POST'])
def showMessageByToken():
  response = {
    'status': '',
    'success': False,
    'message': '',
  }
  token = request.headers.get("token")
  if db.CheckTokenExists(token):
    response['success'] = True
    messages = db.ShowMessageByToken(token)
    if len(messages) != 0:
      response['data'] = messages
    else:
      response['data'] = []
  else:
    response['success'] = False
    response['message'] = "Token invalid."

  return jsonify(response)

@app.route('/showMessageByEmail', methods=['POST'])
def showMessageByEmail():
  response = {
    'status': '',
    'success': False,
    'message': '',
  }
  token = request.headers.get("token")
  data = request.json
  if db.CheckTokenExists(token):
    response['success'] = True
    messages = db.ShowMessageByEmail(data['email'])
    if len(messages) != 0:
      response['data'] = messages
    else:
      response['data'] = []
  else:
    response['success'] = False
    response['message'] = "Token invalid."

  return jsonify(response)

@app.route('/createMessage', methods=['POST'])
def createMessage():
  response = {
    'status': '',
    'success': False,
    'message': '',
  }
  token = request.headers.get("token")
  data = request.json
  sender_id = db.ShowUserIdByToken(token)
  receiver_id = db.ShowUserIdByEmail(data['email'])

  if sender_id is None or receiver_id is None:
    status = 400
    response['success'] = False
    response['message'] = "Fail to create message."
  else:
    status = 200
    db.CreateMessage(sender_id, receiver_id, data['message'], data['location'])
    response['success'] = True
    response['message'] = "Success create message."

  return jsonify(response), status

@app.route('/check')
def check():
  if request.environ.get('wsgi.websocket'):
    ws = request.environ['wsgi.websocket']
    msg = json.loads(ws.receive())
    user = (db.ShowUserInfoByToken(msg['token']))['email']
    logged[user] = ws
    while not ws.closed:
      msg = ws.receive()
      if msg != None:
        msg = json.loads(msg)
        msg = {
          'message': 'Logged.'
        }
        ws.send(json.dumps(msg))
  return 'None'

def run_server():
 app.debug = True
 http_server = WSGIServer(('',5000), app, handler_class=WebSocketHandler)
 http_server.serve_forever()

if __name__ == '__main__':
  run_server()
# app.run(debug=True)
