const API_URL = 'http://localhost:5000';
//const API_URL = 'https://twidder-linkoping.herokuapp.com';

const signUpFunc = function(e){
  e.preventDefault();
  const signUpFormData = {
    firstname: document.getElementById("firstname").value,
    familyname: document.getElementById("lastname").value,
    gender: document.getElementById("selection").value,
    city: document.getElementById("city").value,
    country: document.getElementById("country").value,
    email: document.getElementById("signup-email").value,
    password: document.getElementById("password").value,
  };
  fetch(`${API_URL}/index`, {
      method: "POST",
      headers: new Headers({
        "content-type": "application/json",
      }),
      body: JSON.stringify(signUpFormData),
  })
  .then(function(response) {
    if (response.status !== 200) {
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
      return;
    } else {
      response.json().then((data) => {
        let info = document.getElementById("signUpInfo");
        info.innerHTML = data.message;
      })
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}
const signInFunc = function(e){
  e.preventDefault();
  const signInFormData = {
    email: document.getElementById("email").value,
    password: document.getElementById("signin-pswd").value,
  };
  fetch(`${API_URL}/signIn`, {
    method: "POST",
    headers: new Headers({
      "content-type": "application/json",
    }),
    body: JSON.stringify(signInFormData),
  })
  .then(function(response) {
    if (response.status === 200) {
      response.json().then((data) => {
        if (!data.success) {
          let info = document.getElementById("signInInfo");
          info.innerHTML = data.message;
        } else {
          sessionStorage.setItem('token', data.token);
          loadView();
        }
      })
    } else {
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}

const signOutFunc = function(e){
  e.preventDefault();
  const token = sessionStorage.getItem('token');
  fetch(`${API_URL}/signOut`, {
    method: "POST",
    headers: new Headers({
      "content-type": "application/json",
      token: token,
    }),
  })
  .then(function(response) {
    if (response.status === 200) {
      response.json().then((data) => {
        if (!data.success) {
          console.log(`Looks like there was a problem. Status code: ${data.message}`);
        } else {
          sessionStorage.removeItem('token');  
          loadView();
        }
      })
    } else {
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}

const changePswdFunc = function(e){
  e.preventDefault();
  const token = sessionStorage.getItem('token');
  const changePswdData = {
    password: document.getElementById("old_pswd").value,
    new_password: document.getElementById("new_pswd").value,
  }
  fetch(`${API_URL}/changePassword`, {
    method: "POST",
    headers: new Headers({
      "content-type": "application/json",
      token: token,
    }),
    body: JSON.stringify(changePswdData),
  })
  .then(function(response) {
    if (response.status === 200) {
      response.json().then((data) => {
        if (!data.success) {
          let info = document.getElementById("changeInfo");
          info.innerHTML = data.message;
        } else {
          let info = document.getElementById("changeInfo");
          info.innerHTML = data.message;
          document.getElementById("old_pswd").value = '';
          document.getElementById("new_pswd").value = '';
          document.getElementById("repeat_pswd").value = '';
        }
      })
    } else {
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}

const showUserInfoByToken = function(div){
  const token = sessionStorage.getItem('token');
  fetch(`${API_URL}/showUserInfoByToken`, {
      method: "POST",
      credentials: "include",
      headers: new Headers({
        "content-type": "application/json",
        token: token,
    })
  })
  .then(function(response) {
    if (response.status !== 200) {
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
    } else {
      response.json().then((data) => {
        var panel = document.getElementById("home");
        var userInfo = data;
        var infoDiv = panel.querySelector(".userInfo");
        selectTab(div, "home");
        infoDiv.innerHTML = `<p class='email'>Email: ${userInfo.email}</p>
                            <p>First name: ${userInfo.firstname}</p>
                            <p>Family name: ${userInfo.familyname}</p>
                            <p>Gender: ${userInfo.gender}</p>
                            <p>City: ${userInfo.city}</p>
                            <p>Country: ${userInfo.country}</p>`
        loadWall("home");
      })
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}

const showUserInfoByEmail = function(email){
  const token = sessionStorage.getItem('token');
  let panel = document.getElementById("browse");
  let infoDiv = panel.querySelector(".userInfo");
  fetch(`${API_URL}/showUserInfoByEmail`, {
    method: "POST",
    credentials: "include",
    headers: new Headers({
      "content-type": "application/json",
      token: token,
    }),
    body: JSON.stringify({ email: email }),
  })
  .then(function(response) {
    if (response.status !== 200) {
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
      infoDiv.innerHTML = `<p class='email'>Email: User not found! :sadface</p>
                            <p>First name: </p>
                            <p>Family name: </p>
                            <p>Gender: </p>
                            <p>City: </p>
                            <p>Country: </p>`
    } else {
      response.json().then((data) => {
        var userInfo = data;
        infoDiv.innerHTML = `<p class='email'>Email: ${userInfo['email']}</p>
                            <p>First name: ${userInfo['firstname']}</p>
                            <p>Family name: ${userInfo['familyname']}</p>
                            <p>Gender: ${userInfo['gender']}</p>
                            <p>City: ${userInfo['city']}</p>
                            <p>Country: ${userInfo['country']}</p>`
      })
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}

const showMessageByToken = function(tab){
  const token = sessionStorage.getItem('token');
  let wall = document.getElementById(tab + "_wall");
  wall.innerHTML = "";  
  fetch(`${API_URL}/showMessageByToken`, {
      method: "POST",
      headers: new Headers({
        "content-type": "application/json",
        token: token,
      }),
  })
  .then(function(response) {
    if (response.status !== 200) {
      wall.innerHTML = "User not found :sadface";
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
      return;
    } else {
      response.json().then((data) => {
        var msgs = data.data;
        for (var i = 0; msg = msgs[i++]; ) {
          wall.innerHTML += `<p draggable="true" ondragstart="drag(event)" id="drag${i}-home">${msg.sender_email} @ ${msg.location}: ${msg.message}</p>`;
        }
      })
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}

const showMessageByEmail = function(email){
  const token = sessionStorage.getItem('token');
  let wall = document.getElementById("browse_wall");
  wall.innerHTML = "";  
  fetch(`${API_URL}/showMessageByEmail`, {
    method: "POST",
    headers: new Headers({
      "content-type": "application/json",
      token: token,
    }),
    body: JSON.stringify({ email: email }),
  })
  .then(function(response) {
    if (response.status !== 200) {
      wall.innerHTML = "User not found :sadface";
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
      return;
    } else {
      response.json().then((data) => {
        var msgs = data.data;
        for (var i = 0; msg = msgs[i++]; ) {
          wall.innerHTML += `<p draggable="true" ondragstart="drag(event)" id="drag${i}-home">${msg.sender_email} @ ${msg.location}: ${msg.message}</p>`;
        }
      })
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}

const createMessage = function(email, message, form, tab, loc){
  const token = sessionStorage.getItem('token');
  fetch(`${API_URL}/createMessage`, {
    method: "POST",
    headers: new Headers({
      "content-type": "application/json",
      token: token,
    }),
    body: JSON.stringify({ 
      email: email,
      message: message,
      location: loc,
    }),
  })
  .then(function(response) {
    if (response.status !== 200) {
      var wall = document.getElementById(tab + "_wall");
      wall.innerHTML = "<h1>" + response.message + "</h1>";
      console.log(`Looks like there was a problem. Status code: ${response.status}`);
      return;
    } else {
      response.json().then((data) => {
        form.message.value = "";
        loadWall(tab);
      })
    }
  })
  .catch(function(error) {
    console.log("Fetch error: " + error);
  });
}
